/* Copyright (c) 2009 Ondrej Donek.
   See the file LICENSE.txt for licensing information. */
  
/**
 * Prototype object for single note's category.
 * 
 * @param aArgs {mixed}
 * @type Object
 */
function tbNotesCategoryPrototype(aArgs)
{
  if((aArgs instanceof Components.interfaces.mozIStorageStatementRow)) {
    this._id = aArgs.id;
    this._parent_category_id = aArgs.parent_id;
    this._name = aArgs.name;
    this._description = aArgs.description;
    this._color = aArgs.color;
  }
};
tbNotesCategoryPrototype.prototype = 
{
  // id
  get mId() {
    return (typeof(this._id) != "undefined") ? this._id : null;
  },
  set mId(aVal) {
    delete this._id;
    
    this._id = aVal;
    this._need_update = true;
  },
  
  // parent category id
  get mParentCategoryId() {
    return (typeof(this._parent_category_id) != "undefined") ? this._parent_category_id : null;
  },
  set mParentCategoryId(aVal) {
    delete this._parent_category_id;
    
    this._parent_category_id = aVal;
    this._need_update = true;
  },
  
  // name
  get mName() {
    return (typeof(this._name) != "undefined") ? this._name : null;
  },
  set mName(aVal) {
    delete this._name;
    
    this._name = aVal;
    this._need_update = true;
  },
  
  // description
  get mDescription() {
    return (typeof(this._description) != "undefined") ? this._description : null;
  },
  set mDescription(aVal) {
    delete this._description;
    
    this._description = aVal;
    this._need_update = true;
  },
  
  // color
  get mColor() {
    return (typeof(this._color) != "undefined") ? this._color : null;
  },
  set mColor(aVal) {
    delete this._color;
    
    this._color = aVal;
    this._need_update = true;
  },
  
  // TODO Implement `getParentCategory` method!
  
  
  /**
   * Remove category permanently (from the storage, object remains 
   * how long you will hold it (only id is deleted so mId is null).
   * 
   * @returns {boolean}
   */
  remove: function() {
    if(this.mId != null) {
      if(!tbNotesDb.removeCategory(this.mId)) {
        return false;
      }
    }
    
    delete this._id;
    return true;
  },
  
  /**
   * Returns TRUE if object was changed but not saved.
   *
   * @returns {boolean}
   */
  needSave: function() {
    if(typeof(this._need_update) == "undefined") {
      return false;
    }
    
    return this._need_update;
  },
  
  /**
   * Save/update category's record in the storage. If category hasn't been  
   * saved yet (so id is null) will be category added into the storage and 
   * id updated.
   * 
   * @returns {boolean}
   */
  save: function() {
    if(typeof(this._need_update) != "undefined") {
      delete this._need_update;
    }
    
    if(this.mId == null) {
      var res = tbNotesDb.addCategory(this);
      
      if(res != null) {
        return false;
      } else {
        this._id = res;
        return true;
      }
    } else {
      return res = tbNotesDb.updateCategory(this);
    }
  },
  
  /**
   * Returns object as a string.
   * 
   * @return {string}
   */
  toString: function() {
    var ret = "{ id: \"" + this.mId + "\", " +
              "parent_id: \"" + this.mParentCategoryId + "\", " +
              "name: \"" + this.mName + "\", " + 
              "description: \"" + this.mDescription + "\", " +
              "color: \"" + this.mColor + "\" }";
    
    return ret;
  }
  
}; // End of tbNotesCategoryPrototype
